#[mockall_double::double]
use crate::services::file_store::FileStore;

use self::models::{ArtifactID, CommitArtifact};

pub struct ReleaseManager {
    file_store: FileStore,
}

impl ReleaseManager {
    pub fn new(file_store: FileStore) -> Self {
        Self { file_store }
    }

    pub async fn commit_artifact(&self, request: CommitArtifact) -> anyhow::Result<ArtifactID> {
        tracing::debug!("committing artifact: {:?}", request);
        let artifact_id = ArtifactID::new();

        self.file_store
            .upload_files(artifact_id.clone(), Vec::new())
            .await?;

        // publish domain event that it has been published

        Ok(artifact_id)
    }
}

pub mod extensions;
pub mod models;

#[cfg(test)]
mod test {
    use crate::services::file_store::MockFileStore;

    use super::*;

    #[tokio::test]
    async fn generated_artifact_id() -> anyhow::Result<()> {
        let mut file_store = MockFileStore::new();
        file_store
            .expect_upload_files()
            .times(1)
            .returning(|_, _| Ok(()));

        let releaser_manager = ReleaseManager::new(file_store);

        releaser_manager
            .commit_artifact(CommitArtifact {
                app: "app".into(),
                branch: "branch".into(),
            })
            .await?;

        Ok(())
    }
}
